/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 19/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

#ifndef COLORSCALE_H_
#define COLORSCALE_H_

#include <tulip/Observable.h>
#include <tulip/Color.h>

#include <vector>
#include <map>

namespace tlp {

class TLP_SCOPE ColorScale : public Observable {

public:

  ColorScale();
  ColorScale(const ColorScale& scale);
  virtual ~ColorScale();

  virtual void setColorScale(const std::vector<Color> colors, const bool gradient = true);
  virtual Color getColorAtPos(const float pos) const;

  bool colorScaleInitialized() const {return colorScaleSet;}
  std::map<float, Color> getColorMap() const {return colorMap;}
  bool isGradient(){return gradient;}

protected:

  std::map<float, Color> colorMap;
  bool gradient;
  bool colorScaleSet;
};

}

#endif /* COLORSCALE_H_ */
