/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 19/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#ifndef _Tulip_GLMAINVIEWINTERACTOR_H_
#define _Tulip_GLMAINVIEWINTERACTOR_H_

#include "InteractorChainOfResponsibility.h"

namespace tlp {

  /** \brief Tulip interactor abstract class for NodeLinkDiagramComponent
   *
   */
  class TLP_QT_SCOPE NodeLinkDiagramComponentInteractor  : public InteractorChainOfResponsibility {

  public :

    NodeLinkDiagramComponentInteractor(const QString &iconPath, const QString &text);
    /**
     * return if this interactor is compatible with given View
     */
    virtual bool isCompatible(const std::string &viewName) {
      if(viewName=="Node Link Diagram view")
        return true;
      return false;
    }

  };

  /** \brief Tulip interactor to navigate on the graph
   *
   */
  class TLP_QT_SCOPE InteractorNavigation  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorNavigation();

    /**
     * Construct chain of responsibility
     */
    void construct();

  };

  /** \brief Tulip interactor to get information about an element of the graph
   *
   */
  class TLP_QT_SCOPE InteractorGetInformation  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorGetInformation();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to select par of the graph
   *
   */
  class TLP_QT_SCOPE InteractorSelection  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorSelection();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to move/reshape
   *
   */
  class TLP_QT_SCOPE InteractorSelectionModifier  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorSelectionModifier();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to do a magic selection
   *
   */
  class TLP_QT_SCOPE InteractorMagicSelection  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorMagicSelection();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to do a rectangle zoom
   *
   */
  class TLP_QT_SCOPE InteractorRectangleZoom  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorRectangleZoom();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to delete an element
   *
   */
  class TLP_QT_SCOPE InteractorDeleteElement  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorDeleteElement();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to add a node
   *
   */
  class TLP_QT_SCOPE InteractorAddNode  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorAddNode();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to add edges
   *
   */
  class TLP_QT_SCOPE InteractorAddEdge  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorAddEdge();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

  /** \brief Tulip interactor to edit edge bends
   *
   */
  class TLP_QT_SCOPE InteractorEditEdgeBends  : public NodeLinkDiagramComponentInteractor {

  public:

    /**
     * Default constructor
     */
    InteractorEditEdgeBends();

    /**
     * Construct chain of responsibility
     */
    void construct();
  };

}

#endif






